package Gircap;

use strict;
use warnings;
use Exporter;
use vars qw( @ISA $VERSION @EXPORT @EXPORT_OK);
use English;
use Carp;

my $TRUE=1; my $FALSE = 0;

$VERSION = 1.00;
@ISA = qw(Exporter);



our %CAPABILITY_ORDER = 
    (
     'chown'            =>  0,
     'dac_override'     =>  1,
     'dac_read_search'  =>  2,
     'fowner'           =>  3,
     'fsetid'           =>  4,
     'kill'             =>  5,
     'setgid'           =>  6,
     'setuid'           =>  7,
     'setpcap'          =>  8,
     'linux_immutable'  =>  9,
     'net_bind_service' => 10,
     'net_broadcast'    => 11,
     'net_admin'        => 12,
     'net_raw'          => 13,
     'ipc_lock'         => 14,
     'ipc_owner'        => 15,
     'sys_module'       => 16,
     'sys_rawio'        => 17,
     'sys_chroot'       => 18,
     'sys_ptrace'       => 19,
     'sys_pacct'        => 20,
     'sys_admin'        => 21,
     'sys_boot'         => 22,
     'sys_nice'         => 23,
     'sys_resource'     => 24,
     'sys_time'         => 25,
     'sys_tty_config'   => 26,
     'reserved27'       => 27,
     'reserved28'       => 28,
     'reserved29'       => 29,
     'reserved30'       => 30,
     'reserved31'       => 31,
     );

push(@EXPORT, '%CAPABILITY_ORDER');


our %CAPABILITY_NAME = reverse %CAPABILITY_ORDER; 
push(@EXPORT, '%CAPABILITY_NAME');


sub capabilityBitsFromList($) {
#-----------------------------------------------------------------------------
#  Translate a human-friendly text list of capability names
#  into a bit string that represents those capabilities.  E.g. input
#  'chown,net_raw' would yield output '00000000000000000010000000000001'.
#-----------------------------------------------------------------------------
    my ($inputString) = @_;

    my $negative;
        # boolean: The list is a list of capabilities NOT to be in the list.
    my $nameListString;
        # comma-separated list of capability names

    if (substr($inputString, 0, 1) eq '!') {
        $negative = $TRUE;
        $nameListString = substr($inputString, 1);
    } else {
        $negative = $FALSE;
        $nameListString = $inputString;
    }

    my @capNameList = split(m{,}, $nameListString);

    my $retval;

    $retval = "";
    foreach my $i (0..31) {
        substr($retval, $i, 1) = $negative ? '1' : '0';
    }

    foreach my $capName (@capNameList) {
        my $capabilityOrder = $CAPABILITY_ORDER{$capName};
        
        if (defined($capabilityOrder)) {
            substr($retval, 31-$capabilityOrder, 1) = $negative ? '0' : '1';
        } else {
            die("Unrecognized capability name: '$capName'");
        }
    }
    return $retval;
}

push(@EXPORT, 'capabilityBitsFromList');



sub shorter($$) {
    my ($string1, $string2) = @_;

    return length($string1) < length($string2) ? $string1 : $string2;
}



sub capabilityListFromBits($) {
#-----------------------------------------------------------------------------
#  Translate a capability set bit string to a human-friendly list
#
#  E.g. input of '00000000000000000010000000000001' would yield output of
#  'chown,net_raw'.
#-----------------------------------------------------------------------------
    my ($capBits) = @_;

    if (!defined($capBits)) {
        carp("undefined argument to capabilityListFromBits()");
        return;
    }

    my $retval;

    if ($capBits =~ m{^ [01]{32} $}x) {
        my ($capabilityList, $negCapList);

        $capabilityList = '';   # initial value
        $negCapList = '!';       # initial value
        
        for (my $order = 0; $order < 32; ++$order) {
            my $capabilityBit = substr($capBits, 31 - $order, 1);

            if ($capabilityBit eq '1') {
                if ($capabilityList ne '') {
                    $capabilityList .= ',';
                }
                $capabilityList .= $CAPABILITY_NAME{$order};
            } else {
                if ($negCapList ne '!') {
                    $negCapList .= ',';
                }
                $negCapList .= $CAPABILITY_NAME{$order};
            }
        }
        $retval = shorter($capabilityList, $negCapList);
    }
    return $retval;
}

push(@EXPORT, 'capabilityListFromBits');



# Return success for the "use"
1;
